const path = require('path')
const fse = require('fs-extra')
const klawSync = require('klaw-sync')
const babelParser = require('@babel/parser')
const traverse = require('@babel/traverse').default
const template = require('@babel/template')

/**
 * 解析lib入口文件源码，获取export变量及文件路径
 */
function parseEntry (name, pt, types) {
  const jsonPath = require.resolve(path.join(pt || name, 'package.json')) // package.json路径
  const json = require(jsonPath) // package.json
  const entry = path.join(path.dirname(jsonPath), json.module || json.main) // lib入口文件路径
  const imports = {}
  const exports = {}
  const ast = babelParser.parse(fse.readFileSync(entry, 'utf8'), {
    sourceType: 'module'
  })

  traverse(ast, {
    ImportDeclaration({node}) {
      node.specifiers && node.specifiers.forEach(function (spec) {
        switch (true) {
          case types.isImportSpecifier(spec): // import {xx as yy} from 'source'
          case types.isImportDefaultSpecifier(spec): // import xx from 'source'
            return imports[spec.local.name] = { // yy
              imported: spec.imported.name, // xx
              source: require.resolve(path.join(path.dirname(entry), node.source.value)) // source
            }
        }
      })
    },
    ExportNamedDeclaration({node}) {
      node.specifiers && node.specifiers.forEach(spec => {
        switch (true) {
          case types.isExportSpecifier(spec):
            if (node.source) { // export {xx as yy} from 'source'
              exports[spec.exported.name] = { // yy
                imported: spec.local.name, // xx
                source: require.resolve(path.join(path.dirname(entry), node.source.value)) //source
              }
            } else { // export {xx as yy}
              exports[spec.exported.name] = {
                imported: imports[spec.local.name].imported,
                source: imports[spec.local.name].source
              }
            }

            break
        }
      })

      if (node.declaration) {
        if (types.isFunctionDeclaration(node.declaration)) { // export function fn(){}
          exports[node.declaration.id.name] = {
            imported: node.declaration.id.name,
            source: entry
          }
        } else if (types.isVariableDeclaration(node.declaration)) { // export var xx=xx
          node.declaration.declarations.forEach(function (dec) {
            if (types.isVariableDeclarator(dec)) {
              exports[dec.id.name] = {
                imported: dec.id.name,
                source: entry
              }
            }
          })
        }
      }
    }
  })

  return exports
}

/**
 * 深度优先搜索，查找所有被引用的库文件（直接和间接import），以便加载所有样式文件
 * 因为lib里的文件是rollup打包的，已经tree shaking了，所以我们只要查找import了哪些lib里的文件
 * @param files 直接import的文件
 * @param types
 * @param libDir
 */
function getAllFiles (files, libDir, types) {
  const all = {} // 使用对象去重

  while (files.length) {
    const file = files.pop()
    const ast = babelParser.parse(fse.readFileSync(file, 'utf8'), {
      sourceType: 'module'
    })

    all[file] = true
    traverse(ast, {
      ImportDeclaration({node}) {
        node.specifiers && node.specifiers.forEach(function (spec) {
          switch (true) {
            case types.isImportSpecifier(spec): // import {xx as yy} from 'source'
            case types.isImportDefaultSpecifier(spec): // import xx from 'source'
              if (/^\.\.?\//.test(node.source.value)) { // 相对路径引用为lib里的文件
                const source = require.resolve(path.join(path.dirname(file), node.source.value))

                source.startsWith(libDir) && files.push(source)
              }
          }
        })
      }
    })
  }

  return Object.keys(all)
}

module.exports = function ({types}) {
  let jsonPath  // package.json绝对路径
  let libDir // lib目录绝对路径，即/xxx/xxx/lib
  let exports // lib入口文件的export信息

  return {
    pre() { // 准备工作
      if (!jsonPath) {
        /**
         * 如果配置了path（lib目录绝对路径）就使用path，否则使用require.resolve定位
         * 因为webpack等打包工具可能配置了alias，导致跟require.resolve结果不一致
         */
        jsonPath = require.resolve(path.join(this.opts.path || this.opts.name, 'package.json'))
        libDir = path.dirname(jsonPath)
        exports = parseEntry(this.opts.name, this.opts.path, types)
      }
    },
    visitor: {
      ImportDeclaration(nodePath, {opts: {name, skin, type}}) {
        const {node} = nodePath

        if (node.source.value === name) {
          const imports = [] // 导入的变量
          const map = {} // 导入变量对应的js文件映射
          let isDefaultImport = false

          // 获取import信息
          node.specifiers.forEach(spec => {
            if (types.isImportSpecifier(spec)) { // import {xx as yy} from 'source'
              imports.push({
                imported: spec.imported.name, // xx
                local: spec.local.name // yy
              })
            } else if (types.isImportDefaultSpecifier(spec)) { // import xx from 'source'
              isDefaultImport = true
            }
          })

          // 如果是import xx from 'xx'，啥都不做
          if (isDefaultImport) {
            return
          }

          // 获取每个import变量对应的js文件
          imports.forEach(function ({imported, local}) {
            let source = exports[imported].source

            // 将同一个文件的变量导入放在一个import语句中
            map[source] = map[source] || []
            map[source].push({
              imported: exports[imported].imported,
              local
            })
          })

          // 导入js文件
          let statements = Object.keys(map).map(function (source) {
            return `import {${map[source].map(function ({imported, local}) {
              return imported !== local ? `${imported} as ${local}` : local
            }).join(',')}} from '${source}';`
          })

          // 导入样式文件
          statements = statements.concat(getAllFiles(Object.keys(map), libDir, types).map(function (source) {
            let styles = []
            const styleDir = path.join(path.dirname(source), 'style')

            if (fse.pathExistsSync(styleDir)) { // style目录存在
              let hasPcSuffix // 是否存在xx.pc.js文件

              klawSync(styleDir, {
                nodir: true,    // 忽略目录
                depthLimit: 1   // 递归一次
              }).forEach(function ({path}) {
                if (/\.pc\.js$/.test(path)) {
                  hasPcSuffix = true
                }
              })

              // 根据皮肤配置加载组件对应的样式文件
              switch (true) {
                case skin === 'none': // 不加载皮肤
                  break
                case skin === 'all': // 全部皮肤
                  // 引入style目录下所有样式文件
                  klawSync(styleDir, {
                    nodir: true,  // 忽略目录
                    depthLimit: 1, // 递归一次
                    filter({path}) {
                      return /\.js$/.test(path)
                    }
                  }).forEach(function ({path}) {
                    switch (type) {
                      case 'responsive':  // pc和mobile样式都加载
                        styles.push(path)
                        break
                      case 'pc':
                        if (!hasPcSuffix || /\.pc\.js$/.test(path)) {
                          styles.push(path)
                        }

                        break
                      default:
                        if (!hasPcSuffix || !/\.pc\.js$/.test(path)) {
                          styles.push(path)
                        }
                    }
                  })
                  break
                case skin instanceof Array: // 多个皮肤
                  skin.forEach(function (item) {
                    switch (type) {
                      case 'responsive':
                        styles.push(path.join(styleDir, `${item}.js`))
                        styles.push(path.join(styleDir, `${item}.pc.js`))
                        break
                      case 'pc':
                        styles.push(path.join(styleDir, `${item}${hasPcSuffix ? '.pc' : ''}.js`))
                        break
                      default:
                        styles.push(path.join(styleDir, `${item}.js`))
                    }
                  })
                  break
                default: // 默认皮肤
                  switch (type) {
                    case 'responsive':
                      styles.push(path.join(styleDir, `default.js`))
                      styles.push(path.join(styleDir, `default.pc.js`))
                      break
                    case 'pc':
                      styles.push(path.join(styleDir, `default${hasPcSuffix ? '.pc' : ''}.js`))
                      break
                    default:
                      styles.push(path.join(styleDir, `default.js`))
                  }
              }

              styles = styles.filter(function (style) {
                return fse.pathExistsSync(style)
              }).map(function (style) {
                return `import '${style}';`
              })
            }

            return styles.join('\n')
          }).filter(function (statement) {
            // 过滤空字符串
            return statement
          }))

          // 将一个import语句替换成多个import语句
          nodePath.replaceWithMultiple(template.statements(statements.join('\n'))())
        }
      }
    }
  }
}